import time
from datetime import datetime

from netforce.model import Model, fields, get_model
from netforce.utils import get_data_path
from netforce.access import get_active_user,set_active_user
from netforce.access import get_active_company

class HDcase(Model):
    _name="clinic.hd.case"
    _string="HD Case Treatment"
    _audit_log=True
    _name_field="number"
    _multi_company=True
    
    def get_hrs(self,ids,context={}):
        res={}
        fmt="%Y-%m-%d %H:%M:%S"
        for obj in self.browse(ids):
            diff=datetime.strptime(obj.date_stop,fmt)-datetime.strptime(obj.date_start,fmt)
            total_time=round(diff.seconds/3600,2)
            res[obj.id]=total_time
        return res

    _fields={
        "number": fields.Char("Number",required=True,search=True),
        "patient_id": fields.Many2One("clinic.patient","Patient",required=True,search=True),
        "doctor_id": fields.Many2One("clinic.doctor","Doctor", required=True,search=True),
        "nurse_id": fields.Many2One("clinic.nurse","Nurse", required=True,search=True),
        "date_start": fields.DateTime("Time start",required=True,search=True),
        "date_stop": fields.DateTime("Time stop",required=True,search=True),
        "department_id": fields.Many2One("clinic.department", "Department",search=True),
        "wh_start": fields.Float("Wt.Kg start"),
        "wh_stop": fields.Float("Wt.Kg stop"),
        "bp_start": fields.Integer("BP mmHG start"),
        "per_bp_start": fields.Integer("/Per start"),
        "bp_stop": fields.Integer("BP mmHG stop"),
        "per_bp_stop": fields.Integer("/Per stop"),
        "hct": fields.Integer("HCT %"),
        "state": fields.Selection([("draft","Draft"),("confirmed","Confirmed"),("approved","Approved"),("cancelled","Cancelled"),("paid","Paid")],"Status",required=True),
        "dialyzers": fields.One2Many("clinic.hd.case.dialyzer","hd_case_id","Dializers"),
        "lines": fields.One2Many("clinic.hd.case.line","hd_case_id","Lines"),
        "comments": fields.One2Many("message","related_id","Comments"),
        "company_id": fields.Many2One("company","Company"),
        "fee": fields.Float("HD Fee"),
        "amount": fields.Float("Due Amount",function="get_total",readonly=True,function_multi=True),
        "total": fields.Float("Total",function="get_total",readonly=True,function_multi=True),
        "reconcile_id": fields.Many2One("account.reconcile","Reconcile Id",readonly=True),
        "invoices": fields.One2Many("account.invoice","related_id","Invoices"),
        "pickings": fields.One2Many("stock.picking","related_id","Pickings"),
        "payments": fields.One2Many("account.payment","related_id","Payments"),
        'visit_id': fields.Many2One("clinic.visit", "Visit"),
        'total_time': fields.Integer("Total Time(Hrs)",function="get_hrs"),
        "fee_type": fields.Selection([("mg","Medical Government"),("sc","Social Security"),("nhso","NHSO (30฿)"),("personal","Personal"),("others","Others")],"Fee Type"),
        'fee_partner_id': fields.Many2One("partner","Fee Contact"),
    }

    def _get_number(self,context={}):
        while 1:
            seq_id=get_model("sequence").find_sequence(name="Clinic HD Case")
            num=get_model("sequence").get_next_number(seq_id,context=context)
            if not num:
                return None
            user_id=get_active_user()
            set_active_user(1)
            res=self.search([["number","=",num]])
            set_active_user(user_id)
            if not res:
                return num
            get_model("sequence").increment_number(seq_id,context=context)

    def _get_nurse(self,context={}):
        user_id=get_active_user()
        print("user_id ",user_id)
        nurse_ids=get_model("clinic.nurse").search([['user_id','=',user_id]])
        if nurse_ids:
            return nurse_ids[0]
        return None

    _defaults={
        "state": "draft",
        "date_start": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        "date_stop": lambda *a: time.strftime("%Y-%m-%d %H:%M:%S"),
        'nurse_id': _get_nurse,
        "number": _get_number,
        "company_id": lambda *a: get_active_company(),
        "fee": 1500,
    }
    _order="date_start desc,number desc"

    def onchange_dialyzer(self,context={}):
        data=context["data"]
        path=context["path"]
        line=get_data_path(data,path,parent=True)
        dialyzer_id=line.get("dialzer_id")
        if not dialyzer_id:
            return {}
        dialyzer=get_model("clinic.dialyzer").browse(dialyzer_id)
        line["detail"]=dialyzer.detail
        line["use_time"]=dialyzer.use_time
        line["max_use_time"]=dialyzer.max_use_time
        line["member_type"]=dialyzer.member_type
        line["dialyzer_type"]=dialyzer.dialyzer_type
        line["bid_flow_rate"]=dialyzer.bid_flow_rate
        line["ultrafittration"]=dialyzer.ultrafittration
        line["state"]=dialyzer.state
        return data

    def onchange_patient(self,context={}):
        data=context['data']
        patient_id=data['patient_id']
        hd_cases=self.search_browse([['patient_id','=',patient_id]])
        if hd_cases:
            hd_case=hd_cases[-1]
            data['doctor_id']=hd_case.doctor_id.id
            data['department_id']=hd_case.department_id.id
        else:
            data['doctor_id']=None
            data['department_id']=None
            
        return data

    def onchange_product(self,context={}):
        data=context['data']
        path=context["path"]
        line=get_data_path(data,path,parent=True)
        product_id=line.get('product_id')
        if not product_id:
            return
        prod=get_model("product").browse(product_id)
        line['uom_id']=prod.uom_id.id
        line['description']=prod.name
        line['qty']=1
        line['price']=prod.sale_price or 0.0
        line['amount']=prod.sale_price or 0.0
        self.onchange_line(context)
        return data
    
    def onchange_line(self,context={}):
        data=context['data']
        total=0.0
        for line in data['lines']:
            price=line.get('price') or 0
            qty=line.get('qty') or 0
            amt=qty * price
            product_id=line.get("product_id")
            if product_id:
                line['amount']=amt
            total+=amt
        data['total']=total
        return data

    def cancelled(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({"state":"cancelled"})

    def make_invoices(self,ids,context={}):
        setting=get_model("settings").browse(1)
        currency_id=setting.currency_id.id
        account_receivable_id=setting.account_receivable_id.id
        company_id=get_active_company()
        uom=get_model("uom").search_browse([['name','ilike','%Unit%']])
        if not uom:
            raise Exception("Unit not found in uom")
        obj=self.browse(ids[0])
        due_date=obj.date_start[0:10]
        # XXX
        context['type']='out'
        context['inv_type']='invoice'
        if obj.lines:
            # XXX need to split function prepare invoice
            vals={
                "type": "out",
                "inv_type": "invoice",
                "tax_type": "tax_in",
                'due_date': due_date,
                "ref": obj.number,
                "related_id": "clinic.hd.case,%s"%obj.id,
                "currency_id": currency_id,
                "company_id": company_id,
                "lines": [],
                "company_id": company_id,
            }
            partner=obj.patient_id.partner_id
            if not partner:
                raise Exception("No contact for patient %s"%obj.patient_id.name)
            vals["partner_id"]=partner.id
            for line in obj.lines:
                line={
                    "product_id": line.product_id.id,
                    "description": line.description,
                    "qty": line.qty,
                    "uom_id": line.uom_id.id,
                    "unit_price": line.price,
                    "amount": line.amount,
                    'account_id': line.product_id.sale_account_id.id or account_receivable_id,
                }
                vals['lines'].append(('create',line))
            inv_id=get_model("account.invoice").create(vals,context)
            # create picking
            obj.make_pickings()
        if obj.fee:
            vals={
                "type": "out",
                "inv_type": "invoice",
                "tax_type": "tax_in",
                'due_date': due_date,
                "ref": obj.number,
                "related_id": "clinic.hd.case,%s"%obj.id,
                "currency_id": currency_id,
                "company_id": company_id,
                "lines": [],
                "company_id": company_id,
            }
            vals["partner_id"]=obj.fee_partner_id.id
            if obj.fee_partner_id.account_payable_id:
                account_receivable_id=obj.fee_partner_id.account_payable_id.id
            line={
                "product_id": None,
                "description": "Fee",
                "qty": 1,
                "uom_id": uom[0].id,
                "unit_price": obj.fee,
                "amount": obj.fee,
                'account_id': account_receivable_id,
            }
            vals['lines'].append(('create',line))
            inv_id=get_model("account.invoice").create(vals,context)
    
    def make_pickings(self,ids,context={}):
        obj=self.browse(ids[0])
        # no picking
        if not obj.lines:
            return

        partner=obj.patient_id.partner_id
        if not partner:
            raise Exception("Contact not for this patient")
        ship_address_id=None
        for address in partner.addresses:
            if address.type=="shipping":
                ship_address_id=address.id
                break
        if not ship_address_id:
            raise Exception("contact %s dont'have address with type shipping"%partner.name)
        pick_vals={
            "type": "out",
            "ref": obj.number,
            "related_id": "clinic.hd.case,%s"%obj.id,
            "partner_id": obj.patient_id.partner_id.id,
            "ship_address_id": ship_address_id,
            "state": "draft",
            "lines": [],
        }
        res=get_model("stock.location").search([["type","=","customer"]])
        if not res:
            raise Exception("Customer location not found")
        cust_loc_id=res[0]


        for line in obj.lines:
            prod=line.product_id
            if prod.type != 'stock':
                continue
            wh_loc_id=prod.location_id.id
            if not wh_loc_id:
                res=get_model("stock.location").search([["type","=","internal"]])
                if not res:
                    raise Exception("Warehouse not found")
                wh_loc_id=res[0]
            line_vals={
                "product_id": prod.id,
                "qty": 1,
                "uom_id": prod.uom_id.id,
                "location_from_id": wh_loc_id,
                "location_to_id": cust_loc_id,
            }
            pick_vals["lines"].append(("create",line_vals))
        if not pick_vals["lines"]:
            return {
                "flash": "Nothing left to deliver",
            }
        picking_obj=get_model("stock.picking")
        pick_id=picking_obj.create(pick_vals,context={"pick_type": "out"})
        pick=picking_obj.browse(pick_id)
        pick.set_done([pick_id])

    def post_invoices(self,ids,context={}):
        obj=self.browse(ids[0])
        for inv in obj.invoices:
            inv.post()
        print("Post!") 

    def confirm(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({"state":"confirmed"})

    def paid(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.write({"state":"paid"})
    
    def approve(self,ids,context={}):
        obj=self.browse(ids)[0]
        obj.make_invoices()
        obj.post_invoices()
        obj.write({"state":"approved"})
        return {
            'next': {
                'name': 'clinic_hd_case',
                'mode': 'form',
                'active_id': obj.id,
            },
            'flash': '%s has been approved'%obj.number,
        }

    def approved(self,ids,context={}):
        for obj in self.browse(ids):
            user_id=get_active_user()
            try:
                set_active_user(1)
                settings=get_model("clinic.setting").browse(1)
                reconcile_id=get_model("account.reconcile").create({})
                if obj.patient_id.type == 'mg':
                    vals={
                        "narration": obj.number,
                        "lines": [
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ar_mg_id.id,
                                "debit": obj.amount,
                                "credit": 0,
                                "partner_id": settings.mg_partner_id.id,
                                "due_date": obj.date,
                                "reconcile_id":reconcile_id,
                            }),
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ap_mg_id.id,
                                "debit": 0,
                                "credit": obj.amount,
                            }),
                        ],
                        "state": "posted",
                        "date_posted": obj.date,
                        "date": obj.date,
                    }
                    get_model("account.move").create(vals)
                elif obj.patient_id.type == 'sc':
                    vals={
                        "narration": obj.number,
                        "lines": [
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ar_sc_id.id,
                                "debit": obj.amount,
                                "credit": 0,
                                "partner_id": settings.sc_partner_id.id,
                                "due_date": obj.date,
                                "reconcile_id":reconcile_id,
                            }),
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ap_sc_id.id,
                                "debit": 0,
                                "credit": obj.amount,
                            }),
                        ],
                        "state": "posted",
                        "date_posted": obj.date,
                        "date": obj.date,
                    }
                    get_model("account.move").create(vals)
                elif obj.patient_id.type == 'nhso':
                    vals={
                        "narration": obj.number,
                        "lines": [
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ar_nhso_id.id,
                                "debit": obj.amount,
                                "credit": 0,
                                "partner_id": settings.nhso_partner_id.id,
                                "due_date": obj.date,
                                "reconcile_id":reconcile_id,
                            }),
                            ("create",{
                                "description": obj.number,
                                "account_id": settings.ap_nhso_id.id,
                                "debit": 0,
                                "credit": obj.amount,
                            }),
                        ],
                        "state": "posted",
                        "date_posted": obj.date,
                        "date": obj.date,
                    }
                    get_model("account.move").create(vals)
            finally:
                set_active_user(user_id)
            obj.write({"state":"approved","reconcile_id":reconcile_id})

    def journal_report(self,ids,context={}):
        obj=self.browse(ids[0])
        move_id = get_model("account.move").search([["narration","=",obj.number]])
        if not move_id:
            raise Exception("Order is not post or nove have jounal entry.")
        return {
            "next": {
                "name": "journal_entry",
                "mode":"form",
                "active_id":move_id[0],
            }
        }
    
    def get_total(self,ids,context={}):
        vals={}
        for obj in self.browse(ids):
            total=0
            amt=0
            fee=obj.fee or 0
            for line in obj.lines:
                total+=line.amount or 0
            fee=0 # XXX
            amt=total+fee
            vals[obj.id]={
                "total": total,
                "amount": amt,
            }
        return vals

    def view_hdcase(self,ids,context={}):
        return {
            'name': 'clinic_hd_case',
            'mode': 'form',
            'form_view_xml': 'clinic_hd_case_form',
            'active_id': ids[0],
        }
    
    def delete(self,ids,context={}):
        for obj in self.browse(ids):
            if obj.state != 'draft':
                raise Exception("Can not delete HD Case %s because state is not draft"%obj.number)
        super().delete(ids)
    
    def onchange_fee_type(self,context={}):
        data=context['data']
        fee_type=data.get("fee_type","")
        if fee_type in ("mg","sc","nhso"):
            data['fee']=1500
        else:
            data['fee']=0.0
        return data

HDcase.register()
